/*
 * File name: Circle.java
 * Date:      2014/11/09 19:41
 * Author:    Jan Faigl
 */
package cz.cvut.fel.pr1;

import java.awt.Color;

/**
 *
 * @author jf
 */
public class Circle extends Geom implements Printable {

    private final Point center;
    private final int radius;

    /**
     *
     * @param center
     * @param radius
     */
    public Circle(Point center, int radius) {
        this(center, radius, Color.RED);

    }

    /**
     *
     * @param center
     * @param radius
     * @param color
     */
    public Circle(Point center, int radius, Color color) {
        super(color);
        if (center == null || radius <= 0) {
            throw new IllegalArgumentException();
        }
        this.center = center;
        this.radius = radius;
    }

    /**
     *
     * @param geom
     * @return
     */
    @Override
    public boolean isEqual(Geom geom) {
        boolean ret = geom == this;
        if (!ret && geom instanceof Circle) {
            Circle c = (Circle) geom;
            ret = radius == c.radius && center.isEqual(c.center);
        }
        return ret;
    }

    /**
     *
     * @param pt
     * @return
     */
    public boolean isInside(Point pt) {
        if (pt == null) {
            return false;
        }
        int dx = pt.getX() - center.getX();
        int dy = pt.getY() - center.getY();
        return ((dx * dx + dy * dy) <= radius * radius);
    }

    /**
     *
     * @param sg
     * @return
     */
    public boolean isInside(Segment sg) {
        if (sg == null) {
            return false;
        }
        return isInside(sg.getP0()) && isInside(sg.getP1());
    }

    /**
     *
     * @param sg
     * @return
     */
    public boolean isInside(LineSegment sg) {
        if (sg == null) {
            return false;
        }
        return isInside(sg.getP0()) && isInside(sg.getP1());
    }

    public boolean isInside(Circle c) {
        if (c == null) {
            return false;
        }
        int rd = radius - c.radius;
        if (rd > 0) {
            return new Circle(center, rd).isInside(c.center);
        }
        return false;
    }

    /**
     *
     * @param geom
     * @return
     */
    @Override
    public boolean isInside(Geom geom) {
        if (geom == null) {
            return false;
        }
        boolean ret = this == geom;
        if (!ret && geom instanceof Point) {
            ret = isInside((Point) geom);
        } else if (geom instanceof Segment) {
            ret = isInside((Segment) geom);
        } else if (geom instanceof LineSegment) {
            ret = isInside((LineSegment) geom);
        } else if (geom instanceof Circle) {
            ret = isInside((Circle) geom);
        }
        return ret;
    }

    @Override
    public void printToCanvas(Canvas canvas) {
        if (canvas == null) {
            return;
        }
        Coords[] pts = GridCanvasUtil.drawGridCircle(center, radius);
        if (pts == null) {
            return;
        }
        final int w = canvas.getWidth();
        final int h = canvas.getHeight();

        for (int i = 0; i < pts.length; ++i) {
            Coords pt = pts[i];
            if (pt.getX() >= 0 && pt.getX() < w && pt.getY() >= 0 && pt.getY() < h) {
                canvas.setColorAt(pt.getX(), pt.getY(), color);
            }
        }

    }

    @Override
    public String getShapeName() {
        return "Circle";
    }

}
