/*
 * File name: Segment.java
 * Date:      2014/11/09 19:40
 * Author:    Jan Faigl
 */
package cz.cvut.fel.pr1;

import java.awt.Color;

/**
 *
 * @author jf
 */
public class Segment extends Geom implements Printable {

    private final Point p0;
    private final Point p1;

    /**
     *
     * @param pt1
     * @param pt2
     */
    public Segment(Point pt1, Point pt2) {
        this(pt1, pt2, Color.GREEN);
    }

    /**
     *
     * @param pt1
     * @param pt2
     * @param color
     */
    public Segment(Point pt1, Point pt2, Color color) {
        super(color);
        if (pt1 == null || pt2 == null || pt1.equals(pt2)) {
            throw new IllegalArgumentException();
        }
        p0 = pt1;
        p1 = pt2;
    }

    /**
     *
     * @return
     */
    public Point getP0() {
        return p0;
    }

    /**
     *
     * @return
     */
    public Point getP1() {
        return p1;
    }

    /**
     *
     * @param geom
     * @return
     */
    @Override
    public boolean isEqual(Geom geom) {
        boolean ret = geom == this;
        if (!ret && geom instanceof Segment) {
            Segment s = (Segment) geom;
            ret
                    = (p0.equals(s.p0) && p1.equals(s.p1))
                    || (p1.equals(s.p0) && p0.equals(s.p1));
        }
        return ret;
    }

    /**
     *
     * @param pt
     * @return
     */
    public boolean isInside(Point pt) {
        if (pt == null) {
            return false;
        } else {
            boolean collinear = GeomUtil.wind(p0, p1, pt) == 0;
            return collinear && GeomUtil.inBetween(p0, p1, pt);
        }
    }

    public boolean isInside(Segment s) {
        if (s == null) {
            return false;
        } else {
            return isInside(s.p0) && isInside(s.p1);
        }
    }

    /**
     *
     * @param geom
     * @return
     */
    @Override
    public boolean isInside(Geom geom) {
        if (geom == null) {
            return false;
        }
        boolean ret = this == geom;
        if (!ret && geom instanceof Point) {
            ret = isInside((Point) geom);
        } else if (!ret && geom instanceof Segment) {
            ret = isInside((Segment) geom);
        }
        return ret;
    }

    @Override
    public void printToCanvas(Canvas canvas) {
        if (canvas == null) {
            return;
        }
        Coords[] line = GridCanvasUtil.drawGridLine(p0, p1);
        if (line == null) {
            return;
        }
        final int w = canvas.getWidth();
        final int h = canvas.getHeight();

        for (int i = 0; i < line.length; ++i) {
            Coords pt = line[i];
            if (pt.getX() >= 0 && pt.getX() < w && pt.getY() >= 0 && pt.getY() < h) {
                canvas.setColorAt(pt.getX(), pt.getY(), color);
            }
        }
    }

    @Override
    public String getShapeName() {
        return "Segment";
    }

}
